/*******************************<+>***************************
 **                             PRC
 *************************************************************
 **
 **  $Id: Help.c,v 1.3 1994/02/18 19:56:15 dta Exp $
 **
 **  $Source: /cvs/lib/xn/Help.c,v $
 **
 **  Module Name:
 **
 **  Classification:  Unclassified
 **
 **  Author:          Dale T. Anderson
 **
 *******************************<+>***************************/

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/stat.h>

#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/Label.h>
#include <Xm/Text.h>

#include "Xn.h"

typedef struct {
    Widget shell;
    Widget text;
    char file[256];
} HelpInfo;

static void     accept_cb (Widget w, HelpInfo *hs, XtPointer call)
{
    char           *buf;
    int             fd;

    buf = XmTextGetString (hs->text);

    fd = open (hs->file, O_WRONLY | O_TRUNC | O_CREAT);
    write (fd, buf, strlen (buf));
    close (fd);

    XtFree (buf);
}

static void     confirm_cb (Widget w, HelpInfo *hs, XtPointer call)
{
    XnConfirm (hs->shell, "ARE YOU SURE YOU WISH TO SAVE THIS HELPFILE?",
	(XtCallbackProc) accept_cb, NULL, hs);
}

static void     destroy_cb (Widget w, HelpInfo *hs, XtPointer call)
{
    XtFree ((char *) hs);
}

static char *read_file (HelpInfo *hs, const char *file)
{
    char *dir = getenv ("HELPDIR");
    struct stat st;
    char *buf;
    int fd;

    if (!dir) {
	XnErrorMessage ("Environment variable HELPDIR is not set");
	dir = ".";
    } else if (stat (dir, &st) < 0) {
	XnErrorMessage ("Directory %s does not exist, using current directory", dir);
	dir = ".";
    }

    sprintf (hs->file, "%s/%s", dir, file);

    if (stat (hs->file, &st) < 0) {
	XnErrorMessage ("Can not locate help file %s", file);
    } else {
	fd = open (hs->file, O_RDONLY);
	buf = (char *) XtAlloc (st.st_size + 1);
	read (fd, buf, st.st_size);
	close (fd);
	buf [st.st_size] = 0;
	return buf;
    }
    return NULL;
}

Widget          HelpMsg (Widget parent, const char *file, char *title)
{
    Widget          form, footer;
    char           *buf;
    int             editable;
    XtCallbackProc  ok_cb;

    HelpInfo *hs = XtNew (HelpInfo);

    buf = read_file (hs, file);

    if (!buf)
	buf = "NO HELP AVAILABLE";

    if (title == NULL)
	title = "HELP WINDOW";

    hs->shell = XnCreateShell (file, title, XnPopdown, NULL);
    XnAddCallback (hs->shell, XmNdestroyCallback, destroy_cb, hs);

    form = XtVaCreateWidget ("fontUnits",
	xmFormWidgetClass, hs->shell,
	XmNunitType, Xm100TH_FONT_UNITS,
	NULL);

    editable = (getenv ("MAKEHELPEDITABLE") != NULL);

    if (editable)
	ok_cb = confirm_cb;
    else
	ok_cb = NULL;

    footer = XnCreateFooter (form, ok_cb, NULL, NULL, NULL);

    hs->text = XmCreateScrolledText (form, "uneditable", NULL, 0);

    XtVaSetValues (hs->text,
	XmNrows, 30,
	XmNcolumns, 80,
	XmNeditable, editable,
	XmNeditMode, XmMULTI_LINE_EDIT,
	XmNscrollHorizontal, False,
	XmNvalue, buf,
	XmNtopAttachment, XmATTACH_FORM,
	XmNleftAttachment, XmATTACH_FORM,
	XmNrightAttachment, XmATTACH_FORM,
	XmNbottomAttachment, XmATTACH_WIDGET,
	XmNbottomWidget, footer,
	NULL);

    XtFree (buf);

    XtManageChild (hs->text);
    XtManageChild (form);

    XtPopup (hs->shell, XtGrabNone);

    return hs->shell;
}
