/*******************************<+>***************************
 **                             PRC
 *************************************************************
 **
 **  $Id: Rubber.c,v 1.3 1994/02/13 06:42:45 dta Exp $
 **
 **  $Source: /cvs/lib/xn/Rubber.c,v $
 **
 **  Module Name:
 **
 **  Classification:  Unclassified
 **
 **  Author:          Dale T. Anderson
 **
 *******************************<+>***************************/

#include <stdio.h>
#include <assert.h>
#include <math.h>

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

#include <Xn/Xn.h>

typedef struct {
    XtCallbackProc  proc;
    Widget          w;
    XtPointer       data;
    XnRubberCallbackStruct cb;
}               Rubber;

static Rubber   rubber;

static GC       create_gc (w, color)
Widget          w;
Pixel           color;
{
    XGCValues       values;
    GC              gc;

    XtVaGetValues (w, XtNbackground, &values.background, NULL);
    values.foreground = color;
    values.function = GXxor;
    gc = XCreateGC (XtDisplay (w), XtWindow (w),
		    GCForeground | GCBackground | GCFunction, &values);
    return (gc);
}

static void     press (w, data, event)
Widget          w;
Rubber         *data;
XEvent         *event;
{
    if (data->proc) {
	assert (data->proc);
	data->cb.event = event;
	data->cb.reason = event->type;
	data->cb.button = event->xbutton.button;
	data->cb.x2 = data->cb.x1 = event->xbutton.x;
	data->cb.y2 = data->cb.y1 = event->xbutton.y;
	data->cb.turnon = 1;
	data->proc (w, data->data, &data->cb);
    }
}

static void     nodraw (w, data, event)
Widget          w;
Rubber         *data;
XEvent         *event;
{
    if (data->proc) {
	assert (data->proc);
	data->cb.event = event;
	data->cb.reason = event->type;
	data->cb.turnon = 0;
	data->cb.x2 = event->xmotion.x;
	data->cb.y2 = event->xmotion.y;
	data->proc (w, data->data, &data->cb);
    }
}

static void     motion (w, data, event)
Widget          w;
Rubber         *data;
XEvent         *event;
{
    if (data->proc) {
	assert (data->cb.xorgc);
	assert (data->proc);
	data->cb.event = event;
	data->cb.reason = event->type;
	data->cb.turnon = 0;
	data->proc (w, data->data, &data->cb);
	data->cb.x2 = event->xmotion.x;
	data->cb.y2 = event->xmotion.y;
	data->cb.turnon = 1;
	data->proc (w, data->data, &data->cb);
    }
}

static void     release (w, data, event)
Widget          w;
Rubber         *data;
XEvent         *event;
{
    if (data->proc) {
	assert (data->cb.xorgc);
	assert (data->proc);
	data->cb.event = event;
	data->cb.reason = event->type;
	data->cb.turnon = 0;
	data->cb.x2 = event->xbutton.x;
	data->cb.y2 = event->xbutton.y;
	data->proc (w, data->data, &data->cb);
    }
}

void            XnInitRubber (w, proc, color, data)
Widget          w;
XtCallbackProc  proc;
Pixel           color;
XtPointer       data;
{
    rubber.w = w;
    rubber.cb.xorgc = create_gc (w, color);
    rubber.proc = proc;
    rubber.data = data;

    XtAddEventHandler (w, ButtonPressMask, FALSE,
	(XtEventHandler) press, &rubber);
    XtAddEventHandler (w, ButtonMotionMask, FALSE,
	(XtEventHandler) motion, &rubber);
    XtAddEventHandler (w, ButtonReleaseMask, FALSE,
	(XtEventHandler) release, &rubber);
}

void            XnRemoveRubber ()
{
    if (rubber.w) {
	assert (rubber.cb.xorgc);
	assert (rubber.proc);
	assert (rubber.w);
	XtRemoveEventHandler (rubber.w, ButtonPressMask, FALSE,
	    (XtEventHandler) press, &rubber);
	XtRemoveEventHandler (rubber.w, ButtonMotionMask, FALSE,
	    (XtEventHandler) motion, &rubber);
	XtRemoveEventHandler (rubber.w, ButtonReleaseMask, FALSE,
	    (XtEventHandler) release, &rubber);
	XFreeGC (XtDisplay (rubber.w), rubber.cb.xorgc);
	rubber.w = NULL;
    }
}

void            XnDrawRubberLine (display, window, cb)
Display        *display;
Window          window;
XnRubberCallbackStruct *cb;
{
    XDrawLine (display, window, cb->xorgc,
	       cb->x1, cb->y1, cb->x2, cb->y2);
}

#ifndef ABS
#define ABS(x)	((x > 0) ? (x) : -(x))
#endif

void            XnDrawRubberBox (display, window, cb)
Display        *display;
Window          window;
XnRubberCallbackStruct *cb;
{
    int             dx, dy;

    dx = ABS (cb->x2 - cb->x1);
    dy = ABS (cb->y2 - cb->y1);

    XDrawRectangle (display, window, cb->xorgc,
		    cb->x1 - dx, cb->y1 - dy, dx * 2, dy * 2);
}

void            XnDrawRubberSight (display, window, cb, width, height)
Display        *display;
Window          window;
XnRubberCallbackStruct *cb;
int             width;
int             height;
{
    XDrawLine (display, window, cb->xorgc, cb->x2, 0, cb->x2, height);
    XDrawLine (display, window, cb->xorgc, 0, cb->y2, width, cb->y2);
}

double          XnRubberAngle (cb)
XnRubberCallbackStruct *cb;
{
    double          angle;

    if (cb->x2 - cb->x1 || cb->y2 - cb->y1) {
	angle = atan2 ((double) (cb->y1 - cb->y2), (double) (cb->x2 - cb->x1));
	angle = angle * 180.0 / M_PI;
	while (angle < 0.0)
	    angle += 360.0;
    } else
	angle = 0.0;
    return (angle);
}

double          XnRubberLength (cb)
XnRubberCallbackStruct *cb;
{
    double          dx, dy;

    dx = (cb->x2 - cb->x1);
    dy = (cb->y2 - cb->y1);
    return ((dx || dy) ? sqrt (dx * dx + dy * dy) : 0.0);
}
