/*******************************<+>***************************
 **                             PRC
 *************************************************************
 **
 **  $Id: Working.c,v 1.3 1994/02/18 20:35:57 dta Exp $
 **
 **  $Source: /cvs/lib/xn/Working.c,v $
 **
 **  Module Name:
 **
 **  Classification:  Unclassified
 **
 **  Author:          Dale T. Anderson
 **
 *******************************<+>***************************/

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <assert.h>
#include <sys/stat.h>

#include <Xm/Xm.h>
#include <Xm/MessageB.h>

#include "Xn.h"

static int      _pid = 0;

static Boolean  kill_working (int pid)
{
    kill (pid, SIGKILL);
    return TRUE;
}

static void     destroyCallback (Widget w, int pid, XtPointer call)
{
    if (_pid)
	kill (_pid, SIGKILL);
}

void            XnKillWorkingWindow ()
{
    if (_pid) {
	kill (_pid, SIGKILL);
	_pid = 0;
    }
}

void            XnWorkingWindow (Widget shell, char *msg)
{
    char            working[256];
    char           *argv[10];
    Arg             args[5];
    int             argc;
    char           *progs;
    int             n;
    Position        x, y;
    Dimension       width, height;
    char            geometry[50];
    struct stat     st;

    assert (msg);

    if (_pid)
	return;

    argc = 1;

    if ((progs = getenv ("PROGS"))) {
	strcpy (working, "%s/Warning");
	if (stat (working, &st) < 0)
	    strcpy (working, "/usr/bin/echo");
	else {
	    n = 0;
	    XtSetArg (args[n], XmNx, &x);
	    n++;
	    XtSetArg (args[n], XmNy, &y);
	    n++;
	    XtSetArg (args[n], XmNwidth, &width);
	    n++;
	    XtSetArg (args[n], XmNheight, &height);
	    n++;
	    XtGetValues (shell, args, n);

	    x = x + width / 2 - 150;
	    y = y + height / 2 - 50;

	    if (x <= 0)
		x = 50;
	    if (y <= 0)
		y = 50;

	    sprintf (geometry, "+%d+%d", x, y);
	    argv[argc++] = "-geometry";
	    argv[argc++] = geometry;
	    argv[argc++] = "-title";
	    argv[argc++] = "WORKING";
	}
    } else
	strcpy (working, "/usr/bin/echo");

    argv[0] = working;
    argv[argc++] = msg;
    argv[argc] = 0;

    switch ((_pid = vfork ())) {
	case -1:
	    return;
	case 0:
	    if (msg)
		execvp (working, argv);
	    else
		execvp (working, argv);
	    _exit (0);
    }

    XnAppAddWorkProc (XtWidgetToApplicationContext (shell),
	kill_working, _pid);
    XnAddCallback (shell, XmNdestroyCallback,
	destroyCallback, _pid);
}

Widget          XnWorking (Widget parent, char *msg)
{
    static Widget   dialog = NULL;

    if (!dialog) {
	Arg             args[2];
	XmString        title;

	title = XmStringCreateLtoR ("WORKING", XmSTRING_DEFAULT_CHARSET);
	XtSetArg (args[0], XmNdialogTitle, title);
	XtSetArg (args[1], XmNdialogStyle, XmDIALOG_APPLICATION_MODAL);
	dialog = XmCreateWorkingDialog (parent, "dialog", args, 2);
	XtUnmanageChild (XmMessageBoxGetChild (dialog, XmDIALOG_OK_BUTTON));
	XtUnmanageChild (XmMessageBoxGetChild (dialog, XmDIALOG_HELP_BUTTON));
	XtUnmanageChild (XmMessageBoxGetChild (dialog, XmDIALOG_CANCEL_BUTTON));
	XtUnmanageChild (XmMessageBoxGetChild (dialog, XmDIALOG_SEPARATOR));
	XmStringFree (title);
    }
    if (!msg) {
	XtUnmanageChild (dialog);
    } else {
	XmString        msg;

	msg = XmStringCreateLtoR (msg, XmSTRING_DEFAULT_CHARSET);

	XtVaSetValues (dialog,
		       XmNmessageString, msg,
		       NULL);

	XmStringFree (msg);
	XtManageChild (dialog);
	XmUpdateDisplay (dialog);
	return XtParent (dialog);
    }
    return (0);
}
