/*******************************<+>***************************
 **                     DTA
 *************************************************************
 **
 **  $Id: Pexdb.C,v 1.7 1994/02/18 22:25:54 dta Exp $
 **
 **  $Source: /cvs/progs/pexdb/Pexdb.C,v $
 **
 **  What @ (#):
 **
 **  Author: Dale T. Anderson
 **
 *******************************<+>***************************/
 
#include <stdio.h>
#include <stdio.h>
#include <string.h>
#include <stdarg.h>

#include "Pexdb.h"

#include <Xm/Label.h>
#include <Xm/Text.h>
#include <Xm/Form.h>
#include <Xm/List.h>

static Display *dpy = NULL;

XmStringTable data_items = NULL;
static int data_total = 0;
static int data_count = 0;

int AddToList (Widget w, const char *fmt, ...)
{
    char buffer [256];
    va_list args;

    va_start (args, fmt);
    int status = vsprintf (buffer, fmt, args);
    va_end (args);

    if (data_count >= data_total) {
	data_total += 100;
	if (!data_items)
	    data_items = (XmStringTable) XtMalloc (sizeof (XmString *) * data_total);
	else
	    data_items = (XmStringTable) XtRealloc ((char *) data_items,
		sizeof (XmString *) * data_total);
	assert (data_items);
    }

    XmString item = XmStringCreateLtoR (buffer, XmSTRING_DEFAULT_CHARSET);

    data_items [data_count++] = item;

    return status;
}

static PEXOCData *get_data (PEXStructure id, int element, unsigned long *num_elements)
{
    char *raw_data;
    unsigned long raw_length;

    int float_format = PEXGetProtocolFloatFormat (dpy);
    Status status = PEXFetchElements (dpy, id,
			       PEXBeginning, element, 
			       PEXBeginning, element, 
			       float_format, num_elements,
			       &raw_length, &raw_data);

    if (!status) {
	XnErrorMessage ("Error fetching elements.");
	return NULL;
    }

    PEXOCData *el_data = PEXDecodeOCs (float_format, *num_elements,
	raw_length, raw_data);

    XFree (raw_data);

    return el_data;
}

static void print_element_type (Widget *lists, PEXStructure id, int element)
{
    unsigned long num_elements;
    PEXOCData *el_data = get_data (id, element, &num_elements);

    if (el_data->oc_type == PEXOCExecuteStructure) {
	XnWidgetPrintf (lists[2], "%2d - Execute Structure %d",
	    element, el_data->data.ExecuteStructure.structure);
    } else {
	OpcodeData *data = FindOpcode (el_data->oc_type);

	if (data) {
	    XnWidgetPrintf (lists[2], "%2d - Element [%d]: %s",
		element, data->oc_type, data->name);
	}
    }

    PEXFreeOCData (num_elements, el_data);
}

static void print_element_data (Widget *lists, PEXStructure id, int element)
{
    unsigned long num_elements;
    PEXOCData *el_data = get_data (id, element, &num_elements);

    OpcodeData *data = FindOpcode (el_data->oc_type);

    if (data) {
	if (data->func)
	    data->func (lists[3], el_data);

	XtVaSetValues (lists [3],
	    XmNitems, data_items,
	    XmNitemCount, data_count,
	    NULL);

	data_items = NULL;
	data_count = 0;
	data_total = 0;
    }

    PEXFreeOCData (num_elements, el_data);
}

static void print_ancestors (Widget w, const PEXStructure id)
{
    unsigned long count;

    PEXStructurePath *paths = PEXGetAncestors (dpy, id, PEXTopPart, 0, &count);

    if (paths) {
	for (int i = 0; i < count; i++)
	    for (int j = 0; j < paths [i].count - 1; j++)
		XnWidgetPrintf (w, "Structure %d depth %d",
			paths [i].elements[j].structure, j + 1);

	PEXFreeStructurePaths (count, paths);
    }
}

static void print_structure (Widget *lists, const PEXStructure id)
{
    unsigned long mask = PEXEditMode | PEXElementPtr | PEXHasRefs
			| PEXLengthStructure | PEXNumElements;

    Status status;

    PEXStructureInfo info;
    status = PEXGetStructureInfo (dpy, id,
	PEXGetProtocolFloatFormat (dpy), mask, &info);

    if (!status) {
	XnErrorMessage ("Error Getting Structure Info\n");
	XnPopdown (lists [0]);
	return;
    }

    if (info.has_refs)
	print_ancestors (lists[0], id);

    XnWidgetPrintf (lists[1], "Structure %d", id);

    XnWidgetPrintf (lists[1], "element_count   %d", info.element_count);
    XnWidgetPrintf (lists[1], "size            %d", info.size);
    XnWidgetPrintf (lists[1], "edit_mode       %s",
	 (info.edit_mode == PEXStructureReplace ? "Replace" : "Insert"));
    XnWidgetPrintf (lists[1], "Element Pointer %d", info.element_pointer);

    for (int el = 1; el <= info.element_count; el++)
	print_element_type (lists, id, el);
}

static void load_list (Widget *lists, PEXStructure id)
{
    XmListDeleteAllItems (lists[0]);
    XmListDeleteAllItems (lists[1]);
    XmListDeleteAllItems (lists[2]);
    XmListDeleteAllItems (lists[3]);
    print_structure (lists, id);
}

static PEXStructure get_structure (const char *text)
{
    char *p = strstr (text, "Structure");

    if (p && (p = strchr (p, ' '))) {
	PEXStructure id;

	if (sscanf (p, "%d", &id) != 1 || !id)
	    fprintf (stderr, "Text = %s, id = %d p = %s\n", text, id, p);
	else
	    return id;
    }
    return 0;
}

static void structure_cb (Widget w, Widget *lists, XmListCallbackStruct *cbs)
{
    char *text;

    XmStringGetLtoR (cbs->item, XmSTRING_DEFAULT_CHARSET, &text);

    PEXStructure id = get_structure (text);

    if (id)
	load_list (lists, id);
    else if (w == lists [2] && strstr (text, "Element")) {
	int index;

	if (sscanf (text, "%d", &index) == 1 && index) {
	    XmStringTable items;
	    int count;

	    XtVaGetValues (lists[1],
		XmNitems, &items,
		XmNitemCount, &count,
		NULL);

	    if (count) {
		char *str;
		XmStringGetLtoR (items[0], XmSTRING_DEFAULT_CHARSET, &str);
		id = get_structure (str);

		XmListDeleteAllItems (lists[3]);
		print_element_data (lists, id, index);

		XtFree (str);
	    } else
		XnErrorMessage ("No elements in list 1");
	} else
	    XnErrorMessage ("No index found in %s", index);
    }

    XtFree (text);
}

static Widget create_view (Widget parent, PEXStructure root)
{
    dpy = XtDisplay (parent);

    Widget form = XtVaCreateWidget ("form",
	xmFormWidgetClass, parent,
	XmNfractionBase, 100,
	NULL);

    Widget frame [4];
    static Widget lists [4];

    frame[0] = XnCreateFrame (form, "Ancestors");
    lists[0] = XmCreateScrolledList (frame[0], "list", NULL, 0);

    frame[1] = XnCreateFrame (form, "Structure");
    lists[1] = XmCreateScrolledList (frame[1], "list", NULL, 0);

    frame[2] = XnCreateFrame (form, "Elements");
    lists[2] = XmCreateScrolledList (frame[2], "list", NULL, 0);

    frame[3] = XnCreateFrame (form, "Element Detail");
    lists[3] = XmCreateScrolledList (frame[3], "list", NULL, 0);

    XtVaSetValues (lists[2],
	XmNvisibleItemCount, 16,
	NULL);

    XtVaSetValues (XtParent (frame[1]),
	XmNtopAttachment, XmATTACH_FORM,
	XmNleftAttachment, XmATTACH_FORM,
	XmNbottomAttachment, XmATTACH_POSITION,
	XmNbottomPosition, 30,
	XmNrightAttachment, XmATTACH_POSITION,
	XmNrightPosition, 40,
	NULL);

    XtVaSetValues (XtParent (frame[0]),
	XmNtopAttachment, XmATTACH_POSITION,
	XmNtopPosition, 30,
	XmNleftAttachment, XmATTACH_FORM,
	XmNbottomAttachment, XmATTACH_WIDGET,
	XmNbottomWidget, XtParent (frame [3]),
	XmNrightAttachment, XmATTACH_POSITION,
	XmNrightPosition, 40,
	NULL);

    XtVaSetValues (XtParent (frame[2]),
	XmNtopAttachment, XmATTACH_FORM,
	XmNleftAttachment, XmATTACH_POSITION,
	XmNleftPosition, 40,
	XmNbottomAttachment, XmATTACH_WIDGET,
	XmNbottomWidget, XtParent (frame [3]),
	XmNrightAttachment, XmATTACH_FORM,
	NULL);

    XtVaSetValues (XtParent (frame [3]),
	XmNtopAttachment, XmATTACH_POSITION,
	XmNtopPosition, 60,
	XmNleftAttachment, XmATTACH_FORM,
	XmNbottomAttachment, XmATTACH_FORM,
	XmNrightAttachment, XmATTACH_FORM,
	NULL);

    XnAddCallback (lists[0], XmNdefaultActionCallback, structure_cb, lists);
    XnAddCallback (lists[2], XmNbrowseSelectionCallback, structure_cb, lists);

    print_structure (lists, root);

    XtManageChild (lists[0]);
    XtManageChild (lists[1]);
    XtManageChild (lists[2]);
    XtManageChild (lists[3]);
    XtManageChild (form);

    return form;
}

Widget PexdbWindow (Widget parent, PEXStructure root)
{
    Widget form = XtVaCreateWidget ("form",
	xmFormWidgetClass, parent,
	NULL);

    Widget view = create_view (form, root);

    XtVaSetValues (view,
	XmNtopAttachment, XmATTACH_FORM,
	XmNleftAttachment, XmATTACH_FORM,
	XmNbottomAttachment, XmATTACH_FORM,
	XmNrightAttachment, XmATTACH_FORM,
	NULL);

    XtManageChild (form);

    return form;
}

void PexdbDialog (Widget top, PEXStructure root)
{
    Widget dialog = XmCreateBulletinBoardDialog (top, "dialog", NULL, 0);

    XnWidgetPrintf (XtParent (dialog), "Structure Database Browser");

    PexdbWindow (dialog, root);

    XtManageChild (dialog);

    XtPopup (XtParent (dialog), XtGrabNone);
}
